/***************************************************************
 /"A service by which a client may conduct asynchronous dialogues 
 (message interchanges) with one or more other services. This 
 service is useful when many collaborating services are required 
 to satisfy a client request, and/or when significant delays are 
 involved is satisfying the request. This service was defined 
 under OWS 1.2 in support of SPS operations. WNS has broad 
 applicability in many such multi-service applications.
 
 Copyright (C) 2007 by 52North Initiative for Geospatial 
 Open Source Software GmbH

 Author: Dennis Dahlmann, University of Muenster

 Contact: Andreas Wytzisk, 52North Initiative for Geospatial 
 Open Source Software GmbH,  Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, info@52north.org

 This program is free software; you can redistribute and/or  
 modify it under the terms of the GNU General Public License 
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,  
 but WITHOUT ANY WARRANTY; without even the implied warranty of  
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License 
 along with this program (see gnu-gpl v2.txt); if not, write to  the 
 Free Software Foundation, Inc., 59 Temple Place - Suite 330,  
 Boston, MA 02111-1307, USA or visit the Free Software Foundation's  
 web page, http://www.fsf.org.

 Created on: 2006-07-28

 //Last changes on: 2007-03-15
 //Last changes by: Dennis Dahlmann

 ***************************************************************/
package org.n52.wns.db;

import net.opengis.wns.DoNotificationType;
import net.opengis.wns.GetMessageResponseDocument;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.GDuration;
import org.apache.xmlbeans.XmlException;
import org.n52.wns.WNSDatabaseThread;
import org.n52.wns.WNSException;
import org.n52.wns.WNSInitParamContainer;
import org.n52.wns.WNSServiceException;
import org.x52North.wns.WNSConfigDocument.WNSConfig;
import org.x52North.wns.WNSConfigDocument.WNSConfig.ServiceProperties.DataBaseProperties;
import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Database;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.CollectionManagementService;
import org.xmldb.api.modules.XMLResource;

/**
 * Class to handle MessageDAO request and store them in the eXistDatabase
 * 
 * @author Dennis Dahlmann
 * @version 2.0
 * 
 */
public class ExistDatabaseMessageDAO implements MessageDAO {

	private WNSConfig wnsConfig = null;

	private String dbURL = "";

	private String dbName = "";

	private String dbUser = "";

	private String dbPwd = "";

	private String connectionURL = "";

	private Collection col = null;

	private GDuration maxTTLofMessage = null;

	private WNSDatabaseThread thread = null;

	private static Logger log = Logger.getLogger(ExistDatabaseMessageDAO.class
			.getName());

	/**
	 * This is the constructor, it inits the database to store messages
	 * 
	 * @param properties
	 *            The properties of the database
	 * @throws WNSException
	 */
	public ExistDatabaseMessageDAO(DataBaseProperties properties)
			throws WNSException {
		log.debug("Creating ExistDatabaseMessageDAO instance");
		WNSInitParamContainer initParams = WNSInitParamContainer.getInstance();
		this.wnsConfig = initParams.getWnsConfig();
		this.maxTTLofMessage = this.wnsConfig.getServiceProperties()
				.getMaxTTLOfMessages();
		this.dbURL = this.wnsConfig.getServiceProperties()
				.getDataBaseProperties().getURL();
		this.dbName = this.wnsConfig.getServiceProperties()
				.getDataBaseProperties().getDBName();
		this.dbUser = this.wnsConfig.getServiceProperties()
				.getDataBaseProperties().getUser();
		this.dbPwd = this.wnsConfig.getServiceProperties()
				.getDataBaseProperties().getPassword();

		this.connectionURL = "xmldb:exist://" + this.dbURL + "/xmlrpc" + "/db/"
				+ this.dbName;

		String driver = "org.exist.xmldb.DatabaseImpl";
		Class cl;
		try {
			cl = Class.forName(driver);
			Database database = (Database) cl.newInstance();
			DatabaseManager.registerDatabase(database);
			this.col = DatabaseManager.getCollection(this.connectionURL,
					this.dbUser, this.dbPwd);
			if (this.col == null) {
				Collection root = DatabaseManager.getCollection(
						"xmldb:exist://" + this.dbURL + "/xmlrpc" + "/db",
						this.dbUser, this.dbPwd);
				CollectionManagementService mgtService = (CollectionManagementService) root
						.getService("CollectionManagementService", "1.1");
				this.col = mgtService.createCollection(this.dbName);

			} else {

			}
			log.debug("ExistDatabaseMessageDAO instance successfully build");
		} catch (ClassNotFoundException e) {
			log.fatal("Error during init ExistDatabaseMessageDAO request: "
					+ e.toString());
			throw new WNSException(
					"Error during init ExistDatabaseMessageDAO request: ", e);
		} catch (InstantiationException e) {
			log.fatal("Error during init ExistDatabaseMessageDAO request: "
					+ e.toString());
			throw new WNSException(
					"Error during init ExistDatabaseMessageDAO request: ", e);
		} catch (IllegalAccessException e) {
			log.fatal("Error during init ExistDatabaseMessageDAO request: "
					+ e.toString());
			throw new WNSException(
					"Error during init ExistDatabaseMessageDAO request: ", e);
		} catch (XMLDBException e) {
			log.fatal("Error during init ExistDatabaseMessageDAO request: "
					+ e.toString());
			throw new WNSException(
					"Error during init ExistDatabaseMessageDAO request: ", e);
		}
		this.thread = new WNSDatabaseThread(this.wnsConfig
				.getServiceProperties().getDataBaseProperties()
				.getCheckDBDuration(), this.col);
	}

	/**
	 * @throws WNSServiceException
	 * @see org.n52.wns.db.MessageDAO#getMaxMessageNumber()
	 */
	public int getMaxMessageNumber() throws WNSException {
		log.debug("Trying to get stored messages");
		int id = this.loadMessages();
		log.debug("Found " + id + " stored messages");
		return id;
	}

	/**
	 * @throws WNSServiceException
	 * @see org.n52.wns.db.MessageDAO#getMessage(String mID)
	 */
	public GetMessageResponseDocument getMessage(String mID)
			throws WNSException {
		XMLResource res;
		GetMessageResponseDocument response = GetMessageResponseDocument.Factory
				.newInstance();
		log.debug("Trying to getMessage: " + mID);

		try {
			res = (XMLResource) this.col.getResource(mID);
			if (mID.substring(0, 1).equalsIgnoreCase("n")) {
				net.opengis.wns.GetMessageResponseType mes;
				try {
					mes = net.opengis.wns.GetMessageResponseType.Factory
							.parse(res.getContent().toString());
					response.addNewGetMessageResponse().addNewMessage();
					response.getGetMessageResponse().getMessage().set(mes);
				} catch (XmlException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				// .newInstance();
				// mes.addNewMessage();
				// XmlObject xmlo =(XmlObject) res.getContent();
				// mes.setMessage((Message.Factory.newInstance())
				// res.getContent());

				log.debug("Message successfully retrieved");
			} else {
				log.fatal("MessageID expired: " + mID);
				WNSException we = new WNSException("MessageID expired: " + mID);
				throw we;

			}
		} catch (XMLDBException e) {
			log.fatal(e.toString());
			WNSException we = new WNSException(
					"Error during getMessage request: " + e.toString());
			throw we;
		}
		return response;

	}

	/**
	 * @throws WNSServiceException
	 * @see org.n52.wns.db.MessageDAO#storeNotificationMessage(DoNotificationType
	 *      message, int messageID)
	 */
	public void storeNotificationMessage(DoNotificationType message,
			int messageID) throws WNSException {
		log.debug("Trying to store NotificationMessage");
		XMLResource document;
		try {
			document = (XMLResource) this.col.createResource("N" + messageID,
					"XMLResource");
			document.setContent(message.getMessage().xmlText());
			this.col.storeResource(document);
			this.col.close();
			if (message.isSetMaxTTLOfMessage()
					&& (message.getMaxTTLOfMessage().compareToGDuration(
							this.maxTTLofMessage) < 0)) {
				this.thread.add("N" + messageID, message.getMaxTTLOfMessage());
			} else {
				this.thread.add("N" + messageID, this.maxTTLofMessage);
			}
			log.debug("NotificationMessage stored");
		} catch (XMLDBException e) {
			log.fatal(e.toString());
			WNSException we = new WNSException(
					"Error during storeNotificationMessage request: "
							+ e.toString());
			throw we;
		} catch (WNSException e) {
			log.fatal(e.toString());
			WNSException we = new WNSException(
					"Error during storeNotificationMessage request: "
							+ e.toString());
			throw we;
		}

	}

	private int loadMessages() throws WNSException {
		int messageID = 0;
		try {
			String[] list = this.col.listResources();
			for (String message : list) {
				if (!message.equalsIgnoreCase("savedIDs")) {
					if (new Integer(message.substring(1)) > messageID) {
						messageID = new Integer(message.substring(1));
					}
				}
			}
			return messageID;
		} catch (Exception e) {
			log.fatal("Cannot load stored message numbers: " + e.toString());
			WNSException we = new WNSException(
					"Error during private operation loadMessages request: "
							+ e.toString());
			throw we;

		}
	}

}
